package ServoLib1

  annotation (Coordsys(
      extent=[0, 0; 485, 168],
      grid=[2, 2],
      component=[20, 20]), Window(
      x=0.03,
      y=0.05,
      width=0.48,
      height=0.28,
      library=1,
      autolayout=1));
  model PathPlanning "Generate reference angles for fastest kinematic movement"

    import SI = Modelica.SIunits;

    parameter Integer naxis=2 "number of driven axis";
    parameter Real angleBegDeg[naxis]=zeros(naxis) "start angles in degree";
    parameter Real angleEndDeg[naxis]=zeros(naxis) "end angles in degree";
    parameter SI.AngularVelocity speedMax[naxis]=fill(3, naxis)
      "maximum axis speed";
    parameter SI.AngularAcceleration accMax[naxis]=fill(2.5, naxis)
      "maximum axis acceleration";
    parameter SI.Time startTime=0.5 "start time of movement";

    final parameter Real k=Modelica.Constants.pi/180;
    final parameter SI.Angle angleBeg[:]=angleBegDeg*k "start angles";
    final parameter SI.Angle angleEnd[:]=angleEndDeg*k "end angles";
    final parameter SI.Angle angleRel[:]=angleEnd - angleBeg;
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[1, 1],
        component=[20, 20]),
      Icon(
        Rectangle(extent=[-100, 100; 100, -100], style(color=0, fillColor=7)),
        Text(extent=[-119, 178; 121, 102], string="%name"),
        Polygon(points=[-80, 90; -88, 68; -72, 68; -80, 88; -80, 90], style(
            color=8,
            fillColor=8,
            fillPattern=1)),
        Line(points=[-80, 78; -80, -82], style(color=8)),
        Line(points=[-90, 0; 82, 0], style(color=8)),
        Polygon(points=[90, 0; 68, 8; 68, -8; 90, 0], style(
            color=8,
            fillColor=8,
            fillPattern=1)),
        Text(
          extent=[-42, 55; 29, 12],
          string="w",
          style(color=8)),
        Line(points=[-80, 0; -41, 69; 26, 69; 58, 0], style(color=0))),
      Window(
        x=0.03,
        y=0,
        width=0.88,
        height=0.79),
      Documentation(info="Given

- start and end angles of every axis
- maximum speed of every axis
- maximum acceleration of every axis

this component computes the fastest movement under the
given constraints. This means, that:

1. Every axis accelerates with the maximum acceleration
   until the maximum speed is reached.

2. Drives with the maximum speed as long as possible.

3. Decelerates with the negative of the maximum acceleration
   until rest.

The acceleration, constant velocity and deceleration
phase are determined in such a way that the movement
starts form the start angles and ends at the end angles.

The output of this block are the computed angles as
function of time.
"));
  public
    Modelica.Blocks.Interfaces.InPort refAngles(n=naxis) annotation (extent=[
          100, -10; 120, 10], layer="icon");
    Modelica.Blocks.Sources.KinematicPTP kinematicPTP(
      deltaq=angleRel,
      qd_max=speedMax,
      qdd_max=accMax,
      startTime=startTime) annotation (extent=[-20, -10; 0, 10]);
    Modelica.Blocks.Continuous.Integrator integratorVel(k=ones(naxis))
      annotation (extent=[20, -10; 40, 10]);
    Modelica.Blocks.Continuous.Integrator integratorPos(k=ones(naxis), y0=
          angleBeg) annotation (extent=[60, -10; 80, 10]);
  equation
    connect(kinematicPTP.outPort, integratorVel.inPort) annotation (
        points=[0, 0; 17, 0], style(color=3));
    connect(integratorVel.outPort, integratorPos.inPort) annotation (points=[41
          , 0; 57, 0], style(color=3));
    connect(integratorPos.outPort, refAngles) annotation (points=[82, 0; 109, 0
          ], style(color=3));
  end PathPlanning;
  model Drive "Controller + Motor + Gearbox model"
    import SI = Modelica.SIunits;
    parameter Real kp=10 "gain of position controller";
    parameter Real ks=0.6 "gain of speed controller";
    parameter Real Ts=0.08 "time constant of speed PI controller";
    SI.Angle angleError=feedback1.outPort.signal[1];
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[1, 1],
        component=[20, 20]),
      Icon(
        Rectangle(extent=[-100, 50; 30, -50], style(
            color=1,
            gradient=2,
            fillColor=46,
            fillPattern=1)),
        Rectangle(extent=[30, 10; 100, -10], style(
            gradient=2,
            fillColor=8,
            fillPattern=1)),
        Polygon(points=[-100, -90; -90, -90; -60, -20; -10, -20; 20, -90; 30,
              -90; 30, -100; -100, -100; -100, -90], style(
            color=0,
            gradient=0,
            fillColor=0,
            fillPattern=1)),
        Text(extent=[-100, 118; 100, 60], string="%name")),
      Window(
        x=0.06,
        y=0.04,
        width=0.88,
        height=0.88),
      Diagram(
        Rectangle(extent=[-100, 23; 6, -28], style(color=0, pattern=3)),
        Text(
          extent=[-101, 35; -51, 25],
          string="controller",
          style(color=0)),
        Rectangle(extent=[-26, 80; 54, 43], style(color=0, pattern=3)),
        Text(
          extent=[-42, 90; 8, 80],
          string="motor",
          style(color=0)),
        Rectangle(extent=[57, 80; 95, 43], style(color=0, pattern=3)),
        Text(
          extent=[52, 91; 102, 81],
          string="gear",
          style(color=0))),
      Documentation(info="This is a simple model of a drive train. At the left side
the reference angle is provided as input signal. At the right
side the driven flange of the gearbox is present.
For simplicity, this model ...
"));
    Modelica.Blocks.Interfaces.InPort refAngle annotation (extent=[-120, -10;
          -100, 10], layer="icon");
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation (extent
        =[100, -10; 120, 10], layer="icon");
    Modelica.Blocks.Math.Feedback feedback2 annotation (extent=[-47, -10; -27,
          10]);
    Modelica.Blocks.Math.Gain P(k={kp}) annotation (extent=[-70, -10; -50, 10])
      ;
    Modelica.Blocks.Continuous.PI PI(k={ks}, T={Ts}) annotation (extent=[-20,
          -10; 0, 10]);
    Modelica.Mechanics.Rotational.Torque torque annotation (extent=[0, 50; 20,
          70]);
    Modelica.Mechanics.Rotational.Sensors.AngleSensor angle annotation (extent=
          [80, -60; 100, -40], rotation=-90);
    Modelica.Mechanics.Rotational.Sensors.SpeedSensor speed annotation (extent=
          [40, -60; 60, -40], rotation=-90);
    Modelica.Blocks.Math.Feedback feedback1 annotation (extent=[-96, -10; -76,
          10]);
    Modelica.Mechanics.Rotational.Inertia Jmotor(J=25) annotation (extent=[27,
          50; 47, 70]);
    Modelica.Mechanics.Rotational.SpringDamper spring(c=5.e5, d=500)
      annotation (extent=[60, 50; 80, 70]);
    Modelica.Blocks.Math.Gain gain(k={1.e4}) annotation (extent=[12, -10; 32,
          10]);
  equation
    connect(P.outPort, feedback2.inPort1) annotation (points=[-49, 0
          ; -45, 0]);
    connect(feedback2.outPort, PI.inPort) annotation (points=[-27, 0; -22, 0]);
    connect(speed.outPort, feedback2.inPort2) annotation (points=[50, -60; 50,
          -70; -37, -70; -37, -9], style(color=3));
    connect(angle.outPort, feedback1.inPort2) annotation (points=[90, -61; 90,
          -80; -86, -80; -86, -9], style(color=3));
    connect(feedback1.outPort, P.inPort) annotation (points=[-77, 0; -72, 0],
        style(color=3));
    connect(feedback1.inPort1, refAngle) annotation (points=[-92, 0; -110, 0],
        style(color=3));
    connect(torque.flange_b, Jmotor.flange_a) annotation (points=[20, 60; 27,
          60], style(color=0));
    connect(Jmotor.flange_b, spring.flange_a) annotation (points=[47, 60; 60,
          60], style(color=0));
    connect(spring.flange_b, flange) annotation (points=[80, 60; 90, 60; 90, 0
          ; 105, 0], style(color=0));
    connect(Jmotor.flange_b, speed.flange_a) annotation (points=[47, 60; 50, 60
          ; 50, -40], style(color=0));
    connect(angle.flange_a, Jmotor.flange_b) annotation (points=[90, -41; 90,
          -20; 50, -20; 50, 60; 47, 60], style(color=0));
    connect(PI.outPort, gain.inPort) annotation (points=[1, 0; 9, 0], style(
          color=3));
    connect(gain.outPort, torque.inPort) annotation (points=[33, 0; 40, 0; 40,
          30; -20, 30; -20, 60; -2, 60], style(color=3));
  end Drive;
  model RobotStructure
    import SI = Modelica.SIunits;
    parameter SI.Mass loadMass=20;
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[1, 1],
        component=[20, 20]),
      Icon(
        Rectangle(extent=[-100, 80; 100, -100], style(color=0, fillColor=8)),
        Text(extent=[-114, 152; 114, 82], string="%name"),
        Polygon(points=[-80, -61; -32, 33; -18, 27; -66, -66; -80, -61], style(
            color=0,
            gradient=3,
            fillColor=49)),
        Polygon(points=[-21, 41; 81, 61; 84, 47; -19, 27; -21, 41], style(
            color=0,
            gradient=3,
            fillColor=49)),
        Ellipse(extent=[-34, 42; -12, 20], style(gradient=3, fillColor=9)),
        Ellipse(extent=[-84, -56; -62, -78], style(gradient=3, fillColor=9)),
        Rectangle(extent=[68, 67; 94, 39], style(gradient=3, fillColor=10)),
        Rectangle(extent=[-92, -78; 92, -87], style(color=0, fillColor=0))),
      Window(
        x=0.2,
        y=0.06,
        width=0.76,
        height=0.88));
    Modelica.Mechanics.Rotational.Interfaces.Flange_a axis1 annotation (extent=
          [-120, -90; -100, -70]);
    Modelica.Mechanics.Rotational.Interfaces.Flange_a axis2 annotation (extent=
          [-120, 30; -100, 50]);
    ModelicaAdditions.MultiBody.Parts.InertialSystem inertial annotation (
        extent=[-100, -20; -80, 0]);
    ModelicaAdditions.MultiBody.Joints.Revolute r1 annotation (extent=[-38, 0;
          -18, -20]);
    ModelicaAdditions.MultiBody.Parts.BoxBody body1(
      r={1,0,0},
      InnerWidth=0.08,
      InnerHeight=0.08) annotation (extent=[-9, -20; 11, 0]);
    ModelicaAdditions.MultiBody.Joints.Revolute r2 annotation (extent=[18, -20
          ; 38, 0]);
    ModelicaAdditions.MultiBody.Parts.BoxBody body2(
      r={1,0,0},
      InnerWidth=0.08,
      InnerHeight=0.08,
      Material={0,1,0,0.5}) annotation (extent=[50, -20; 70, 0]);
    ModelicaAdditions.MultiBody.Parts.Body load(m=loadMass) annotation (extent=
          [80, -20; 100, 0]);
    ModelicaAdditions.MultiBody.Parts.FrameRotation FrameRotation1(n={0,1,0},
        angle=20) annotation (extent=[-70, -20; -50, 0]);
  equation
    connect(r1.frame_b, body1.frame_a) annotation (points=[-17.5, -
          10; -9.5, -10]);
    connect(body1.frame_b, r2.frame_a) annotation (points=[11.5, -10; 17.5, -10
          ]);
    connect(r2.frame_b, body2.frame_a) annotation (points=[38.5, -10; 49.5, -10
          ]);
    connect(body2.frame_b, load.frame_a) annotation (points=[70.5, -10; 79.5, -
          10]);
    connect(r2.axis, axis2) annotation (points=[28, -2; 28, 40; -114, 40],
        style(color=0));
    connect(r1.axis, axis1) annotation (points=[-28, -16; -28, -80; -108, -80]
        , style(color=0));
    connect(inertial.frame_b, FrameRotation1.frame_a) annotation (points=[-79,
          -10; -69, -10]);
    connect(FrameRotation1.frame_b, r1.frame_a) annotation (points=[-50, -10; -
          39, -10]);
  end RobotStructure;

  model SampledPI
    extends Modelica.Blocks.Interfaces.BlockIcon;
    parameter Real k=1 "Gain";
    parameter Real T(min=Modelica.Constants.eps) = 1 "Time Constant";
    parameter Real startTime=0;
    parameter Real sampleTime;
    discrete output Real x "State of block";
    discrete output Real u "sampled input signal";
    discrete output Real y "sampled output signal";

    Boolean sampleTrigger=sample(startTime, sampleTime);
    annotation (Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[1, 1],
        component=[20, 20]), Window(
        x=0.19,
        y=0.11,
        width=0.61,
        height=0.8));
    Modelica.Blocks.Interfaces.OutPort outPort(final n=1) annotation (extent=[
          100, -10; 120, 10]);
    Modelica.Blocks.Interfaces.InPort inPort(final n=1) annotation (extent=[
          -140, -20; -100, 20]);
  equation
    u = inPort.signal[1];
    y = outPort.signal[1];

    when {initial(),sampleTrigger} then
      x = pre(x) + (sampleTime/T)*u;
      y = k*(x + u);
    end when;
  end SampledPI;
  model SampledDrive "Controller + Motor + Gearbox model"
    import SI = Modelica.SIunits;
    parameter Real kp=10 "gain of position controller";
    parameter Real ks=0.6 "gain of speed controller";
    parameter Real Ts=0.08 "time constant of speed PI controller";
    parameter Real sampleTime;
    SI.Angle angleError=feedback1.outPort.signal[1];
    annotation (
      Coordsys(
        extent=[-100, -100; 100, 100],
        grid=[1, 1],
        component=[20, 20]),
      Icon(
        Rectangle(extent=[-100, 50; 30, -50], style(
            color=1,
            gradient=2,
            fillColor=46,
            fillPattern=1)),
        Rectangle(extent=[30, 10; 100, -10], style(
            gradient=2,
            fillColor=8,
            fillPattern=1)),
        Polygon(points=[-100, -90; -90, -90; -60, -20; -10, -20; 20, -90; 30,
              -90; 30, -100; -100, -100; -100, -90], style(
            color=0,
            gradient=0,
            fillColor=0,
            fillPattern=1)),
        Text(extent=[-100, 118; 100, 60], string="%name")),
      Window(
        x=0.01,
        y=0.01,
        width=0.88,
        height=0.88),
      Diagram(
        Rectangle(extent=[-100, 23; 6, -28], style(color=0, pattern=3)),
        Text(
          extent=[-101, 35; -51, 25],
          string="controller",
          style(color=0)),
        Rectangle(extent=[-26, 80; 54, 43], style(color=0, pattern=3)),
        Text(
          extent=[-42, 90; 8, 80],
          string="motor",
          style(color=0)),
        Rectangle(extent=[57, 80; 95, 43], style(color=0, pattern=3)),
        Text(
          extent=[52, 91; 102, 81],
          string="gear",
          style(color=0))),
      Documentation(info="This is a simple model of a drive train. At the left side
the reference angle is provided as input signal. At the right
side the driven flange of the gearbox is present.
For simplicity, this model ...
"));
    Modelica.Blocks.Interfaces.InPort refAngle annotation (extent=[-120, -10;
          -100, 10], layer="icon");
    Modelica.Mechanics.Rotational.Interfaces.Flange_b flange annotation (extent
        =[100, -10; 120, 10], layer="icon");
    Modelica.Blocks.Math.Feedback feedback2 annotation (extent=[-47, -10; -27,
          10]);
    Modelica.Blocks.Math.Gain P(k={kp}) annotation (extent=[-70, -10; -50, 10])
      ;
    Modelica.Mechanics.Rotational.Torque torque annotation (extent=[0, 50; 20,
          70]);
    Modelica.Mechanics.Rotational.Sensors.AngleSensor angle annotation (extent=
          [80, -60; 100, -40], rotation=-90);
    Modelica.Mechanics.Rotational.Sensors.SpeedSensor speed annotation (extent=
          [40, -60; 60, -40], rotation=-90);
    Modelica.Blocks.Math.Feedback feedback1 annotation (extent=[-96, -10; -76,
          10]);
    Modelica.Mechanics.Rotational.Inertia Jmotor(J=25) annotation (extent=[27,
          50; 47, 70]);
    Modelica.Mechanics.Rotational.SpringDamper spring(c=5.e5, d=500)
      annotation (extent=[60, 50; 80, 70]);
    Modelica.Blocks.Math.Gain gain(k={1.e4}) annotation (extent=[12, -10; 32,
          10]);
    SampledPI sampledPI(
      k=ks,
      T=Ts,
      sampleTime=sampleTime) annotation (extent=[-20, -10; 0, 10]);
  equation
    connect(P.outPort, feedback2.inPort1) annotation (points=[-49, 0
          ; -45, 0]);
    connect(speed.outPort, feedback2.inPort2) annotation (points=[50, -60; 50,
          -70; -37, -70; -37, -9], style(color=3));
    connect(angle.outPort, feedback1.inPort2) annotation (points=[90, -61; 90,
          -80; -86, -80; -86, -9], style(color=3));
    connect(feedback1.outPort, P.inPort) annotation (points=[-77, 0; -72, 0],
        style(color=3));
    connect(feedback1.inPort1, refAngle) annotation (points=[-92, 0; -110, 0],
        style(color=3));
    connect(torque.flange_b, Jmotor.flange_a) annotation (points=[20, 60; 27,
          60], style(color=0));
    connect(Jmotor.flange_b, spring.flange_a) annotation (points=[47, 60; 60,
          60], style(color=0));
    connect(spring.flange_b, flange) annotation (points=[80, 60; 90, 60; 90, 0
          ; 105, 0], style(color=0));
    connect(Jmotor.flange_b, speed.flange_a) annotation (points=[47, 60; 50, 60
          ; 50, -40], style(color=0));
    connect(angle.flange_a, Jmotor.flange_b) annotation (points=[90, -41; 90,
          -20; 50, -20; 50, 60; 47, 60], style(color=0));
    connect(gain.outPort, torque.inPort) annotation (points=[33, 0; 40, 0; 40,
          30; -20, 30; -20, 60; -2, 60], style(color=3));
    connect(feedback2.outPort, sampledPI.inPort) annotation (points=[-28, 0;
          -23, 0], style(color=3));
    connect(sampledPI.outPort, gain.inPort) annotation (points=[2, 0; 9, 0],
        style(color=3));
  end SampledDrive;

end ServoLib1;
