package SolutionsExercise1 "Example solutions of exercise 1 of Advanced Modelica Tutorial at Modelica'2002"
  import Modelica.SIunits;

model VolumeCaseIa
  SIunits.Mass M(start=30.0) "mass";
  SIunits.InternalEnergy E(start=30*2.8e6) "Inner Energy";
  SIunits.MassFlowRate mdot_in "mass flow rate";
  SIunits.MassFlowRate mdot_out "mass flow rate";
  SIunits.SpecificEnthalpy h_in "specific enthalpy at inflow";
  SIunits.SpecificEnthalpy h(start=2.8e6, stateSelect=StateSelect.never)
    "specific enthalpy";
  SIunits.Pressure p(start=60.0e5, stateSelect=StateSelect.never) "pressure";
  SIunits.Density rho "density";
  SIunits.Power Q "heat flow into volume";
  parameter SIunits.Power Q0=5.0e5 "max heat flow";
  parameter SIunits.Volume V=1.0 "volume";
  annotation (
    experiment(StopTime=50.0, Tolerance=1e-6),
    Icon(
      Text(
        extent=[-66, 74; 64, 28],
        string="Case Ia",
        style(color=0)),
      Text(
        extent=[-60, 22; 60, -4],
        string="States M, E",
        style(color=0)),
      Text(
        extent=[-94, -6; 92, -34],
        string="Initialize with states",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.2,
      y=0.02,
      width=0.6,
      height=0.6),
    Documentation(info="<HTML>
	      <h4>Model description</h4>
	      <p>This is the base case model, using the text-book form of the equations also for simulation.
	      <p> It has to be initialized with initial values for the mass and internal energy, M and E.
	      These are not usually known from e.g. process design, so this is not the preferred form of the
	      model. Furthermore, a non-linear system of equations has to be solved for calculating p and h.
<p>This model was originally presented at the Modelica Tutorial in Munich, March 2002</p>
</HTML>
"));
equation
  // mass - and energy balance
  der(M) = mdot_in + mdot_out;
  der(E) = mdot_in*h_in + mdot_out*h + Q;
  // relation between intensive and extensive variables
  E = V*rho*(h - p/rho);
  M = V*rho;
  rho = fsteam(p, h);
  // boundary conditions, very simple ones
  mdot_in = 0.0;
  mdot_out = 1.0*sin(0.2*time);
  h_in = 2.7e6;
  Q = Q0*sin(time);
end VolumeCaseIa;

model VolumeCaseIIa
  SIunits.Mass M(start=30.0) "total mass in volume";
  SIunits.InternalEnergy E(start=30*2.8e6) "inner energy";
  SIunits.MassFlowRate mdot_in "mass flow rate";
  SIunits.MassFlowRate mdot_out "mass flow rate";
  SIunits.SpecificEnthalpy h_in "specific enthalpy at inflow";
  SIunits.SpecificEnthalpy h(start=2.8e6, stateSelect=StateSelect.prefer)
    "specific enthalpy";
  SIunits.Pressure p(start=60.0e5, stateSelect=StateSelect.prefer) "pressure";
  SIunits.Density rho "density";
  SIunits.Power Q "heat flow into volume";
  parameter SIunits.Power Q0=5.0e5 "max heat flow";
  parameter SIunits.Volume V=1.0 "volume";
  annotation (
    experiment(StopTime=50.0, Tolerance=1e-6),
    Icon(
      Text(
        extent=[-66, 76; 64, 30],
        string="Case IIa",
        style(color=0)),
      Text(
        extent=[-60, 24; 60, -2],
        string="States p, h",
        style(color=0)),
      Text(
        extent=[-94, -4; 92, -32],
        string="Initialize with states",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.35,
      y=0.03,
      width=0.32,
      height=0.48),
    Documentation(info="<HTML>
	      <h4>Model description</h4>
	      <p>This is the the same as the base case model, using the text-book form of the equations, but
	      they are not used in theis form for simulation. By using the stateSelect attribute, pressure p
	      and specific enthalpy h are used as states instead. When they are choosen as states, they are
	      also used as defaults for the initial values.
	      <p> This model is easier to initialize because p and ha are usually known from the Mollier diagram
	      and the steady state plant values.
<p>This model was originally presented at the Modelica Tutorial in Munich, March 2002</p>
</HTML>
"));
equation
  // mass - and energy balance
  der(M) = mdot_in + mdot_out;
  der(E) = mdot_in*h_in + mdot_out*h + Q;
  // relation between intensive and extensive variables
  E = V*rho*(h - p/rho);
  M = V*rho;
  rho = fsteam(p, h);
  // boundary conditions, very simple ones
  mdot_in = 0.0;
  mdot_out = 1.0*sin(0.2*time);
  h_in = 2.7e6;
  Q = Q0*sin(time);
end VolumeCaseIIa;

model VolumeCaseIb
  SIunits.Mass M(start=30.0) "mass";
  SIunits.InternalEnergy E(start=30*2.8e6) "Inner Energy";
  SIunits.MassFlowRate mdot_in "mass flow rate";
  SIunits.MassFlowRate mdot_out "mass flow rate";
  SIunits.SpecificEnthalpy h_in "specific enthalpy at inflow";
  SIunits.SpecificEnthalpy h(start=2.8e6, stateSelect=StateSelect.never)
    "specific enthalpy";
  SIunits.Pressure p(start=60.0e5, stateSelect=StateSelect.never) "pressure";
  SIunits.Density rho "density";
  SIunits.Power Q "heat flow into volume";
  parameter SIunits.Power Q0=5.0e5 "max heat flow";
  parameter SIunits.Volume V=1.0 "volume";
  annotation (
    experiment(StopTime=50.0, Tolerance=1e-6),
    Icon(
      Text(
        extent=[-70, 74; 60, 28],
        string="Case Ib",
        style(color=0)),
      Text(
        extent=[-64, 22; 56, -4],
        string="States M, E",
        style(color=0)),
      Text(
        extent=[-108, 0; 108, -42],
        string="Initialize in steady state",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.48,
      y=0.11,
      width=0.32,
      height=0.48),
    Documentation(info="<HTML>
	      <h4>Model description</h4>
	      <p>This is the base case model, using the text-book form of the equations also for simulation.
	      A non-linear system of equations has to be solved for calculating p and h.
	      <p> Initialization is specified explicitely to be in steady state in this model.
	      When start values are given, they are used as initial guesses.
<p>This model was originally presented at the Modelica Tutorial in Munich, March 2002</p>
</HTML>
"));
initial equation
  der(p) = 0.0;
  der(h) = 0.0;
  // mass - and energy balance
equation
  der(M) = mdot_in + mdot_out;
  der(E) = mdot_in*h_in + mdot_out*h + Q;
  // relation between intensive and extensive variables
  E = V*rho*(h - p/rho);
  M = V*rho;
  rho = fsteam(p, h);
  // boundary conditions, very simple ones
  mdot_in = 0.0;
  mdot_out = 1.0*sin(0.2*time);
  h_in = 2.7e6;
  Q = Q0*sin(time);
end VolumeCaseIb;

model VolumeCaseIIb
  SIunits.Mass M(start=30.0) "mass";
  SIunits.InternalEnergy E(start=30*2.8e6) "Inner Energy";
  SIunits.MassFlowRate mdot_in "mass flow rate";
  SIunits.MassFlowRate mdot_out "mass flow rate";
  SIunits.SpecificEnthalpy h_in "specific enthalpy at inflow";
  SIunits.SpecificEnthalpy h(start=2.8e6, stateSelect=StateSelect.always)
    "specific enthalpy";
  SIunits.Pressure p(start=60.0e5, stateSelect=StateSelect.always) "pressure";
  SIunits.Density rho "density";
  SIunits.Power Q "heat flow into volume";
  parameter SIunits.Power Q0=5.0e5 "max heat flow";
  parameter SIunits.Volume V=1.0 "volume";
  annotation (
    experiment(StopTime=50.0, Tolerance=1e-6),
    Icon(
      Text(
        extent=[-70, 74; 60, 28],
        string="Case IIb",
        style(color=0)),
      Text(
        extent=[-64, 22; 56, -4],
        string="States p, h",
        style(color=0)),
      Text(
        extent=[-108, 0; 108, -42],
        string="Initialize in steady state",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.17,
      y=0.32,
      width=0.32,
      height=0.48),
    Documentation(info="<HTML>
	      <h4>Model description</h4>
	      <p>This is the the same as the base case model, using the text-book form of the equations, but
	      they are not used in theis form for simulation. By using the stateSelect attribute, pressure p
	      and specific enthalpy h are used as states instead.
	      <p> Initialization is specified explicitely to be in steady state in this model.
	      When start values are given, they are used as initial guesses.
<p>This model was originally presented at the Modelica Tutorial in Munich, March 2002</p>
</HTML>
"));
initial equation
  der(p) = 0.0;
  der(h) = 0.0;
  // mass - and energy balance
equation
  der(M) = mdot_in + mdot_out;
  der(E) = mdot_in*h_in + mdot_out*h + Q;
  // relation between intensive and extensive variables
  E = V*rho*(h - p/rho);
  M = V*rho;
  rho = fsteam(p, h);
  // boundary conditions, very simple ones
  mdot_in = 0.0;
  mdot_out = 1.0*sin(0.2*time);
  h_in = 2.7e6;
  Q = Q0*sin(time);
end VolumeCaseIIb;

function fsteam "linear approximation to steam table"
  input SIunits.Pressure p;
  input SIunits.SpecificEnthalpy h;
  output SIunits.Density rho;
protected
  parameter SIunits.Pressure p0=60.0e5;
  parameter SIunits.SpecificEnthalpy h0=2.8e6;
  parameter SIunits.Density rho0=30.29526;
  parameter Real ap=5.115157e-6;
  parameter Real ah=-3.345309e-5 "linearization parameters of steamtable ";
protected
  Real delp;
  Real delh;
  annotation (
    derivative=fsteam_der,
    Icon(Text(
        extent=[-78, 62; 76, 18],
        string="Property function",
        style(color=0)), Text(
        extent=[-64, 20; 72, -14],
        string="rho = f(p,h)",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.33,
      y=0.3,
      width=0.32,
      height=0.48),
Documentation(info="<HTML>
	      <h4>function description</h4>
	      <p>This is a linear approximation to the IF97-version of the steam tables at the point
	      p = 60 bars, h = 2.8e3kJ/kg.
</HTML>
"));
algorithm
  delp := p - p0;
  delh := h - h0;
  rho := rho0 + ap*delp + ah*delh;
end fsteam;

function fsteam_der "derivative of linear approximation to steam table"
  input SIunits.Pressure p;
  input SIunits.SpecificEnthalpy h;
  input Real dp "pressure derivative";
  input Real dh "enthalpy derivative";
  output Real drho;
protected
  parameter SIunits.Pressure p0=60.0e5;
  parameter SIunits.SpecificEnthalpy h0=2.8e6;
  parameter SIunits.Density rho0=30.29526;
  parameter Real ap=5.115157e-6;
  parameter Real ah=-3.345309e-5 "linearization parameters of steamtable";
  annotation (
    Icon(Text(
        extent=[-100, 74; 98, 14],
        string="Property function derivative",
        style(color=0)), Text(
        extent=[-82, 26; 84, -16],
        string="rho = f(p,h,der(p),der(h))",
        style(color=0))),
    Coordsys(
      extent=[-100, -100; 100, 100],
      grid=[2, 2],
      component=[20, 20]),
    Window(
      x=0.28,
      y=0.25,
      width=0.6,
      height=0.6),
Documentation(info="<HTML>
	      <h4>function description</h4>
	      <p>This is the derivative of the  linear approximation to the IF97-version of the steam tables at the point
	      p = 60 bars, h = 2.8e3kJ/kg.
</HTML>
"));
algorithm
  drho := ap*dp + ah*dh;
end fsteam_der;

annotation (
  Coordsys(
    extent=[0, 0; 832, 581],
    grid=[2, 2],
    component=[20, 20]),
  Window(
    x=0.11,
    y=0.14,
    width=0.6,
    height=0.6,
    library=1,
    autolayout=1),
  Documentation(info="<HTML>

<h4>Package description</h4>
This package contains a simple example for demonstrating Modelicas new state selection features.
<h4>Version Info and Revision history
</h4>
<address>Author: Hubertus Tummescheit, <br>
Lund University<br>
Department of Automatic Control<br>
Box 118, 22100 Lund, Sweden<br>
email: hubertus@control.lth.se
</address>
<ul>
<li>Initial version: March 14 2002</li>
</ul>

<h4>Sources for model and Literature:</h4>
			      The model has been used in Hilding Elmqvists PhD-thesis
			      (1978, Department of Automatic Control, Lund University)
			      to explain the numerical consequences of state selection.

<h4>Known limits of validity:</h4>
			      The model has been designed to be simple and demonstrative. An easy extension would be  to discretize the model in space.
<h4>Tips for usage and initialization:</h4>
No known problems.

<p>This package was originally presented at the Modelica Tutorial in Munich, March 2002</p>
</HTML>
"));
end SolutionsExercise1;

